diff --git a/src/backend/storage/ipc/dsm_impl.c b/src/backend/storage/ipc/dsm_impl.c
index 70f899e765..4af16381e9 100644
--- a/src/backend/storage/ipc/dsm_impl.c
+++ b/src/backend/storage/ipc/dsm_impl.c
@@ -377,6 +377,9 @@ dsm_impl_posix(dsm_op op, dsm_handle handle, Size request_size,
 
 	/* Map it. */
 	address = mmap(NULL, request_size, PROT_READ | PROT_WRITE,
+#ifdef MAP_ALIGNED_SUPER
+				   MAP_ALIGNED_SUPER |
+#endif
 				   MAP_SHARED | MAP_HASSEMAPHORE | MAP_NOSYNC, fd, 0);
 	if (address == MAP_FAILED)
 	{
diff --git a/src/backend/utils/mmgr/dsa.c b/src/backend/utils/mmgr/dsa.c
index 3b7e2ad126..ae5d3be290 100644
--- a/src/backend/utils/mmgr/dsa.c
+++ b/src/backend/utils/mmgr/dsa.c
@@ -63,9 +63,10 @@
  * The size of the initial DSM segment that backs a dsa_area created by
  * dsa_create.  After creating some number of segments of this size we'll
  * double this size, and so on.  Larger segments may be created if necessary
- * to satisfy large requests.
+ * to satisfy large requests.  2MB is chosen to match the size of huge pages
+ * on common hardware (though not many OSes currently try to use them).
  */
-#define DSA_INITIAL_SEGMENT_SIZE ((size_t) (1 * 1024 * 1024))
+#define DSA_INITIAL_SEGMENT_SIZE ((size_t) (2 * 1024 * 1024))
 
 /*
  * How many segments to create before we double the segment size.  If this is
@@ -2090,9 +2091,8 @@ make_new_segment(dsa_area *area, size_t requested_pages)
 	 * we wouldn't need to use FreePageManager).
 	 *
 	 * We decide on a total segment size first, so that we produce tidy
-	 * power-of-two sized segments.  This is a good property to have if we
-	 * move to huge pages in the future.  Then we work back to the number of
-	 * pages we can fit.
+	 * power-of-two sized segments to align with operating system pages.
+	 * Then we work back to the number of pages we can fit.
 	 */
 	total_size = DSA_INITIAL_SEGMENT_SIZE *
 		((size_t) 1 << (new_index / DSA_NUM_SEGMENTS_AT_EACH_SIZE));
